include machine.e
include scientific.e

-- Test code:
----------------------------------
constant 
MAX_FLOAT64 = {#FF,#FF,#FF,#FF,#FF,#FF,#EF,#7F },
MIN_DENORMALIZED_FLOAT64 = {#01, #00, #00, #00, #00, #00, #00, #00},
MIN_NORMALIZED_FLOAT64 = {#00, #00, #00, #00, #00, #00, #10, #00}

atom max_atom, min_normalized_atom, min_denormalized_atom
max_atom = float64_to_atom( MAX_FLOAT64 )
min_denormalized_atom = float64_to_atom( MIN_DENORMALIZED_FLOAT64 )
min_normalized_atom = float64_to_atom( MIN_NORMALIZED_FLOAT64 )

constant
TEST = {
	{"2.5e-2", 2.5e-2 * 0 + (25 / 1000) },
	{"-1.0e-3",-1.0e-3},
	{"1.5e+0", 1.5e+0 },
	{ "1e0", 1e0 },
	{"1.5e1", 1.5e+1 },
	{"2.5e-1", 2.5e-1 },
	{"3.33e-3",3.33e-3},
	{"-4.02509e+6",-4.02509e+6},
	{"1.234567890123456e+100",1.234567890123456e+100},
	{sprintf("%0.16e", max_atom), max_atom},
	{sprintf("%0.16e", min_normalized_atom), min_normalized_atom},
	{sprintf("%0.16e", min_denormalized_atom), min_denormalized_atom}	
	}

procedure display_atom_bits( atom a )
	sequence b
	integer exp
	b = bytes_to_bits( atom_to_float64( a ))
	exp = bits_to_int( b[53..63] ) - 1023
	b += '0'
	printf(1,"%s %s %s (%d)\n", {b[1..52], b[53..63], b[64], exp  })
end procedure

procedure test( sequence test_vals )
	atom t
	sequence f64, cf64
	for i = 1 to length( test_vals ) do
		cf64 = atom_to_float64( test_vals[i][2] )
		f64 = scientific_to_float64( test_vals[i][1] )
		t = float64_to_atom( f64 )
		if equal( cf64, f64 ) then
			printf(1, "Passed: %20s %0.20e\n", {test_vals[i][1],t}) 
			display_atom_bits( test_vals[i][2] )
			display_atom_bits( t )
			puts(1, "\n")
		else
			printf(1, "\nFailed: %s\nDifference: %0.16e\n", {test_vals[i][1], t - test_vals[i][2] })
			printf(1,"Control: %0.16e\n", {test_vals[i][2]} )
			printf(1,"Test:    %0.16e\n", { t } )
			display_atom_bits( test_vals[i][2] )
			display_atom_bits( t )
			puts(1, "Control ?: " ) ? test_vals[i][2]
			puts(1, "Test    ?: " ) ? t
			puts( 1, "\n")
		end if
		
	end for
end procedure
test( TEST )
